@@ -23,13 +23,13 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include "../include/wpe/view-backend-exportable-egl.h"
+
 #include "exported-buffer-shm-private.h"
 #include "linux-dmabuf/linux-dmabuf.h"
 #include "view-backend-exportable-fdo-egl-private.h"
-#include "view-backend-exportable-private.h"
+#include "view-backend-private.h"
 #include "ws-egl.h"
-#include "wpe/view-backend-exportable-egl.h"
-
 #include <epoxy/egl.h>
 #include <cassert>
 #include <list>
@@ -293,7 +293,7 @@ wpe_view_backend_exportable_fdo_egl_create(const struct wpe_view_backend_exporta
     else
         clientBundle.reset(new ClientBundleEGLDeprecated(client, data, nullptr, width, height));
 
-    struct wpe_view_backend* backend = wpe_view_backend_create_with_backend_interface(&view_backend_exportable_fdo_interface, clientBundle.get());
+    struct wpe_view_backend* backend = wpe_view_backend_create_with_backend_interface(&view_backend_private_interface, clientBundle.get());
 
     return new struct wpe_view_backend_exportable_fdo(std::move(clientBundle), backend);
 }
