@@ -102,7 +102,7 @@ static const struct wl_surface_interface s_surfaceInterface = {
     [](struct wl_client* client, struct wl_resource* surfaceResource, uint32_t callback)
     {
         auto& surface = *static_cast<Surface*>(wl_resource_get_user_data(surfaceResource));
-        if (!surface.exportableClient)
+        if (!surface.apiClient)
             return;
 
         struct wl_resource* callbackResource = wl_resource_create(client, &wl_callback_interface, 1, callback);
@@ -163,6 +163,22 @@ static const struct wl_compositor_interface s_compositorInterface = {
 };
 
 static const struct wpe_bridge_interface s_wpeBridgeInterface = {
+    // initialize
+    [](struct wl_client*, struct wl_resource* resource)
+    {
+        uint32_t implementationType = WPE_BRIDGE_CLIENT_IMPLEMENTATION_TYPE_WAYLAND;
+        switch (Instance::singleton().impl().type()) {
+        case ImplementationType::EGL:
+        case ImplementationType::EGLStream:
+        case ImplementationType::SHM:
+            implementationType = WPE_BRIDGE_CLIENT_IMPLEMENTATION_TYPE_WAYLAND;
+            break;
+        default:
+            break;
+        }
+
+        wpe_bridge_send_implementation_info(resource, implementationType);
+    },
     // connect
     [](struct wl_client*, struct wl_resource* resource, struct wl_resource* surfaceResource)
     {
@@ -505,20 +521,20 @@ void Instance::releaseAudioPacketExport(struct wpe_audio_packet_export* packet_e
     wpe_audio_packet_export_send_release(packet_export->exportResource);
 }
 
-void Instance::registerViewBackend(uint32_t bridgeId, ExportableClient& exportableClient)
+void Instance::registerViewBackend(uint32_t bridgeId, APIClient& apiClient)
 {
     auto it = m_viewBackendMap.find(bridgeId);
     if (it == m_viewBackendMap.end())
         g_error("Instance::registerViewBackend(): " "Cannot find surface with bridgeId %" PRIu32 " in view backend map.", bridgeId);
 
-    it->second->exportableClient = &exportableClient;
+    it->second->apiClient = &apiClient;
 }
 
 void Instance::unregisterViewBackend(uint32_t bridgeId)
 {
     auto it = m_viewBackendMap.find(bridgeId);
     if (it != m_viewBackendMap.end()) {
-        it->second->exportableClient = nullptr;
+        it->second->apiClient = nullptr;
         m_viewBackendMap.erase(it);
     }
 }
@@ -531,8 +547,8 @@ void Instance::unregisterSurface(Surface* surface)
         });
     if (it != m_viewBackendMap.end()) {
         m_viewBackendMap.erase(it);
-        if (surface->exportableClient)
-            surface->exportableClient->bridgeConnectionLost(it->first);
+        if (surface->apiClient)
+            surface->apiClient->bridgeConnectionLost(it->first);
     }
 }
 
